@@ -1,8 +1,7 @@
-# -*- coding: utf-8 -*-
 from collections import OrderedDict
+from http import HTTPStatus
 
 import pytest
-import sys
 import yaml
 
 from apispec import APISpec, BasePlugin
@@ -15,6 +14,7 @@ from apispec.exceptions import (
 
 from .utils import (
     get_schemas,
+    get_examples,
     get_paths,
     get_parameters,
     get_responses,
@@ -140,38 +140,38 @@ class TestComponents:
 
     def test_schema(self, spec):
         spec.components.schema("Pet", {"properties": self.properties})
-        defs = get_schemas(spec)
-        assert "Pet" in defs
-        assert defs["Pet"]["properties"] == self.properties
+        schemas = get_schemas(spec)
+        assert "Pet" in schemas
+        assert schemas["Pet"]["properties"] == self.properties
 
     def test_schema_is_chainable(self, spec):
         spec.components.schema("Pet", {"properties": {}}).schema(
             "Plant", {"properties": {}}
         )
-        defs = get_schemas(spec)
-        assert "Pet" in defs
-        assert "Plant" in defs
+        schemas = get_schemas(spec)
+        assert "Pet" in schemas
+        assert "Plant" in schemas
 
     def test_schema_description(self, spec):
         model_description = "An animal which lives with humans."
         spec.components.schema(
             "Pet", {"properties": self.properties, "description": model_description}
         )
-        defs = get_schemas(spec)
-        assert defs["Pet"]["description"] == model_description
+        schemas = get_schemas(spec)
+        assert schemas["Pet"]["description"] == model_description
 
     def test_schema_stores_enum(self, spec):
         enum = ["name", "photoUrls"]
         spec.components.schema("Pet", {"properties": self.properties, "enum": enum})
-        defs = get_schemas(spec)
-        assert defs["Pet"]["enum"] == enum
+        schemas = get_schemas(spec)
+        assert schemas["Pet"]["enum"] == enum
 
     def test_schema_discriminator(self, spec):
         spec.components.schema(
             "Pet", {"properties": self.properties, "discriminator": "name"}
         )
-        defs = get_schemas(spec)
-        assert defs["Pet"]["discriminator"] == "name"
+        schemas = get_schemas(spec)
+        assert schemas["Pet"]["discriminator"] == "name"
 
     def test_schema_duplicate_name(self, spec):
         spec.components.schema("Pet", {"properties": self.properties})
@@ -181,6 +181,26 @@ class TestComponents:
         ):
             spec.components.schema("Pet", properties=self.properties)
 
+    def test_response(self, spec):
+        response = {"description": "Pet not found"}
+        spec.components.response("NotFound", response)
+        responses = get_responses(spec)
+        assert responses["NotFound"] == response
+
+    def test_response_is_chainable(self, spec):
+        spec.components.response("resp1").response("resp2")
+        responses = get_responses(spec)
+        assert "resp1" in responses
+        assert "resp2" in responses
+
+    def test_response_duplicate_name(self, spec):
+        spec.components.response("test_response")
+        with pytest.raises(
+            DuplicateComponentNameError,
+            match='Another response with name "test_response" is already registered.',
+        ):
+            spec.components.response("test_response")
+
     def test_parameter(self, spec):
         parameter = {"format": "int64", "type": "integer"}
         spec.components.parameter("PetId", "path", parameter)
@@ -207,25 +227,28 @@ class TestComponents:
         ):
             spec.components.parameter("test_parameter", "path")
 
-    def test_response(self, spec):
-        response = {"description": "Pet not found"}
-        spec.components.response("NotFound", response)
-        responses = get_responses(spec)
-        assert responses["NotFound"] == response
+    # Referenced examples are only supported in OAS 3.x
+    @pytest.mark.parametrize("spec", ("3.0.0",), indirect=True)
+    def test_example(self, spec):
+        spec.components.example("test_example", {"value": {"a": "b"}})
+        examples = get_examples(spec)
+        assert examples["test_example"]["value"] == {"a": "b"}
 
-    def test_response_is_chainable(self, spec):
-        spec.components.response("resp1").response("resp2")
-        responses = get_responses(spec)
-        assert "resp1" in responses
-        assert "resp2" in responses
+    @pytest.mark.parametrize("spec", ("3.0.0",), indirect=True)
+    def test_example_is_chainable(self, spec):
+        spec.components.example("test_example_1", {}).example("test_example_2", {})
+        examples = get_examples(spec)
+        assert "test_example_1" in examples
+        assert "test_example_2" in examples
 
-    def test_response_duplicate_name(self, spec):
-        spec.components.response("test_response")
+    @pytest.mark.parametrize("spec", ("3.0.0",), indirect=True)
+    def test_example_duplicate_name(self, spec):
+        spec.components.example("test_example", {})
         with pytest.raises(
             DuplicateComponentNameError,
-            match='Another response with name "test_response" is already registered.',
+            match='Another example with name "test_example" is already registered.',
         ):
-            spec.components.response("test_response")
+            spec.components.example("test_example", {})
 
     def test_security_scheme(self, spec):
         sec_scheme = {"type": "apiKey", "in": "header", "name": "X-API-Key"}
@@ -253,6 +276,21 @@ class TestComponents:
         spec.components.schema("Pet", properties=self.properties, enum=enum)
         assert spec.to_dict() == yaml.safe_load(spec.to_yaml())
 
+    def test_components_can_be_accessed_by_plugin_in_init_spec(self):
+        class TestPlugin(BasePlugin):
+            def init_spec(self, spec):
+                spec.components.schema(
+                    "TestSchema",
+                    {"properties": {"key": {"type": "string"}}, "type": "object"},
+                )
+
+        spec = APISpec(
+            "Test API", version="0.0.1", openapi_version="2.0", plugins=[TestPlugin()]
+        )
+        assert get_schemas(spec) == {
+            "TestSchema": {"properties": {"key": {"type": "string"}}, "type": "object"}
+        }
+
 
 class TestPath:
     paths = {
@@ -550,12 +588,7 @@ class TestPath:
                 == metadata["components"]["responses"]["test_response"]
             )
 
-    @pytest.mark.skipif(
-        int(sys.version[0]) == 2, reason="HTTPStatus only available in Python3"
-    )
     def test_response_with_HTTPStatus_code(self, spec):
-        from http import HTTPStatus
-
         code = HTTPStatus(200)
         spec.path(
             path="/pet/{petId}",
