@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 import json
 
 import pytest
@@ -10,6 +8,7 @@ from marshmallow import Schema
 from apispec import APISpec
 from apispec.ext.marshmallow import MarshmallowPlugin
 from apispec.ext.marshmallow.openapi import MARSHMALLOW_VERSION_INFO
+from apispec.ext.marshmallow import common
 from apispec.exceptions import APISpecError
 from .schemas import (
     PetSchema,
@@ -43,7 +42,8 @@ class TestDefinitionHelper:
     @pytest.mark.parametrize("schema", [AnalysisSchema, AnalysisSchema()])
     def test_resolve_schema_dict_auto_reference(self, schema):
         def resolver(schema):
-            return schema.__name__
+            schema_cls = common.resolve_schema_cls(schema)
+            return schema_cls.__name__
 
         spec = APISpec(
             title="Test auto-reference",
@@ -77,7 +77,8 @@ class TestDefinitionHelper:
     )
     def test_resolve_schema_dict_auto_reference_in_list(self, schema):
         def resolver(schema):
-            return schema.__name__
+            schema_cls = common.resolve_schema_cls(schema)
+            return schema_cls.__name__
 
         spec = APISpec(
             title="Test auto-reference",
@@ -150,6 +151,32 @@ class TestDefinitionHelper:
         pet_exclude = definitions["MultiModifierSchema"]["properties"]["pet_exclude"]
         assert pet_exclude == build_ref(spec, "schema", "Pet_Exclude")
 
+    def test_schema_instance_with_different_modifers_custom_resolver(self):
+        class MultiModifierSchema(Schema):
+            pet_unmodified = Nested(PetSchema)
+            pet_exclude = Nested(PetSchema(partial=True))
+
+        def resolver(schema):
+            schema_instance = common.resolve_schema_instance(schema)
+            prefix = "Partial-" if schema_instance.partial else ""
+            schema_cls = common.resolve_schema_cls(schema)
+            name = prefix + schema_cls.__name__
+            if name.endswith("Schema"):
+                return name[:-6] or name
+            return name
+
+        spec = APISpec(
+            title="Test Custom Resolver for Partial",
+            version="0.1",
+            openapi_version="2.0",
+            plugins=(MarshmallowPlugin(schema_name_resolver=resolver),),
+        )
+
+        with pytest.warns(None) as record:
+            spec.components.schema("NameClashSchema", schema=MultiModifierSchema)
+
+        assert len(record) == 0
+
     def test_schema_with_clashing_names(self, spec):
         class Pet(PetSchema):
             another_field = String()
@@ -293,6 +320,13 @@ class TestCustomField:
         assert props_b["name"]["format"] == "int32"
 
 
+def get_nested_schema(schema, field_name):
+    try:
+        return schema._declared_fields[field_name]._schema
+    except AttributeError:
+        return schema._declared_fields[field_name]._Nested__schema
+
+
 class TestOperationHelper:
     @pytest.mark.parametrize(
         "pet_schema",
@@ -443,7 +477,9 @@ class TestOperationHelper:
             name = parameter["name"]
             assert description == PetSchema.description[name]
         post = p["post"]
-        post_schema = spec_fixture.openapi.resolve_schema_dict(PetSchema)
+        post_schema = spec_fixture.marshmallow_plugin.resolver.resolve_schema_dict(
+            PetSchema
+        )
         assert (
             post["requestBody"]["content"]["application/json"]["schema"] == post_schema
         )
@@ -522,6 +558,55 @@ class TestOperationHelper:
             "schema"
         ] == build_ref(spec, "schema", "Pet")
 
+    @pytest.mark.parametrize(
+        "pet_schema", (PetSchema, PetSchema(), "tests.schemas.PetSchema"),
+    )
+    def test_schema_name_resolver_returns_none_v2(self, pet_schema):
+        def resolver(schema):
+            return None
+
+        spec = APISpec(
+            title="Test resolver returns None",
+            version="0.1",
+            openapi_version="2.0",
+            plugins=(MarshmallowPlugin(schema_name_resolver=resolver),),
+        )
+        spec.path(
+            path="/pet",
+            operations={"get": {"responses": {200: {"schema": pet_schema}}}},
+        )
+        get = get_paths(spec)["/pet"]["get"]
+        assert "properties" in get["responses"]["200"]["schema"]
+
+    @pytest.mark.parametrize(
+        "pet_schema", (PetSchema, PetSchema(), "tests.schemas.PetSchema"),
+    )
+    def test_schema_name_resolver_returns_none_v3(self, pet_schema):
+        def resolver(schema):
+            return None
+
+        spec = APISpec(
+            title="Test resolver returns None",
+            version="0.1",
+            openapi_version="3.0.0",
+            plugins=(MarshmallowPlugin(schema_name_resolver=resolver),),
+        )
+        spec.path(
+            path="/pet",
+            operations={
+                "get": {
+                    "responses": {
+                        200: {"content": {"application/json": {"schema": pet_schema}}}
+                    }
+                }
+            },
+        )
+        get = get_paths(spec)["/pet"]["get"]
+        assert (
+            "properties"
+            in get["responses"]["200"]["content"]["application/json"]["schema"]
+        )
+
     @pytest.mark.parametrize("spec_fixture", ("2.0",), indirect=True)
     def test_schema_uses_ref_in_parameters_and_request_body_if_available_v2(
         self, spec_fixture
@@ -727,16 +812,16 @@ class TestOperationHelper:
         }
 
     def test_schema_global_state_untouched_2json(self, spec_fixture):
-        assert RunSchema._declared_fields["sample"]._Nested__schema is None
+        assert get_nested_schema(RunSchema, "sample") is None
         data = spec_fixture.openapi.schema2jsonschema(RunSchema)
         json.dumps(data)
-        assert RunSchema._declared_fields["sample"]._Nested__schema is None
+        assert get_nested_schema(RunSchema, "sample") is None
 
     def test_schema_global_state_untouched_2parameters(self, spec_fixture):
-        assert RunSchema._declared_fields["sample"]._Nested__schema is None
+        assert get_nested_schema(RunSchema, "sample") is None
         data = spec_fixture.openapi.schema2parameters(RunSchema)
         json.dumps(data)
-        assert RunSchema._declared_fields["sample"]._Nested__schema is None
+        assert get_nested_schema(RunSchema, "sample") is None
 
 
 class TestCircularReference:
