@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Utilities to get elements of generated spec"""
 
 from apispec.utils import build_reference
@@ -10,16 +9,20 @@ def get_schemas(spec):
     return spec.to_dict()["components"]["schemas"]
 
 
+def get_responses(spec):
+    if spec.openapi_version.major < 3:
+        return spec.to_dict()["responses"]
+    return spec.to_dict()["components"]["responses"]
+
+
 def get_parameters(spec):
     if spec.openapi_version.major < 3:
         return spec.to_dict()["parameters"]
     return spec.to_dict()["components"]["parameters"]
 
 
-def get_responses(spec):
-    if spec.openapi_version.major < 3:
-        return spec.to_dict()["responses"]
-    return spec.to_dict()["components"]["responses"]
+def get_examples(spec):
+    return spec.to_dict()["components"]["examples"]
 
 
 def get_security_schemes(spec):
